
; 6502 Badge Software
; Hardware design by Lee Hart
; Software design by Daryl Rictor
;
; This software is free to use and modify in any non-commercial application.
; Comericial use is prohibited without expressed, written permission from the author.
;
;-------------------------------
; LED driver for 6502 Badge     |
;                               |
; by Daryl Rictor  Apr 14, 2019 |
;-------------------------------
	
; also defined in buttons.asm
btn_act		=	$58		; button's down if bit=1
btn_ack		=	$59		; button ackowleged if bit=0
	
LbuffD		=	$2A0		; font storage for LED data pins
LbuffA		=	$2C0		; font storage for LED address pins
Lscanc		=	$2E0
Lscan		=	$2E1
Lptr		= 	$E2		; LED pointer
Ldig		=	$E3		; digit counter
LSptr		=	$E4		; LED address pointer (2 bytes)
Lscl		=	$E6		; scroll speed
Lsclc		=	$E7		; scroll counter
LEDchk1		=	$E8		; config checksum 1
LEDchk2		=	$E9		; config checksum 2

LSCROLL		=	$20		; constant scroll speed
LSCAN		=	$20

Ldef 		.byte	"6502 Badge for VCF-MW 2020      " ; default message 32 bytes

;----------------------------------------------------------------------------
LED_Init	lda	LEDchk1		; check if we have already initialized the video buffer
		cmp	#$A5		; first checksum byte = $A5
		bne	Linit2		; if not a match, we will initialize the checksums and buffer
		lda	LEDchk2		;
		cmp	#$5A		; second checksum is $5A
		beq	Linit3		; match, initialize just buffer pointers and shift counter 
Linit2		lda	#$A5		; init first checksum
		sta	LEDchk1		;
		lda	#$5A		; init second checksum
		sta	LEDchk2		; 
		ldy	#$1F		; store default display in buffer 32 bytes
Linit1		lda	Ldef,y		; text
		and	#$7F		; 0-95 (after subtraction)
		SEC			; set carry for subtraction
		sbc	#$20		; adjust ASCII code
		asl			; mult *2
		tax			; set index
		lda	FONT_TABLE,x	; get address font
		sta	LbuffA,y	; save it
		inx			; next byte
		lda	FONT_TABLE,x	; get data font
		sta	LbuffD,y	; save it
		dey			; get next character
		bpl	Linit1		; loop until all 32 done 
Linit3		STZ	Lptr		; clear pointer
		stz	LSptr		; low byte of pointer
		lda	#LSCROLL	; set high byte scroll delay
		sta	Lscl		; save it	
		sta	Lsclc		; clear scroll counter 
		lda	#LSCAN		;
		sta	Lscan		; save it	
		sta	Lscanc		; clear counter 
		stz	btn_act		; no active buttons
		stz	btn_ack		; no button unacknowleged
		rts			; done

;----------------------------------------------------------------------------
; call this to input a text message and convert it to the font codes
;
SLED_cmd	ldx	#$00		; buffer counter
Sled1		lda	Buffer, y	; get first character
		and	#$7F		; 0-95 (after subtraction)
		CMP	#$0D		; is it EOL
		beq	Sled_eol	; yes
		phy			; save buffer pointer
		sec			; adjust ASCII
		sbc	#$20		; - 32
		asl			; mult *2
		tay			; set index
		lda	FONT_TABLE,y	; get address font
		sta	LbuffA,x	; save it
		iny			; next byte
		lda	FONT_TABLE,y	; get data font
		sta	LbuffD,x	; save it
		ply			; restore buffer pointer
		iny			; get next character
		inx			; inc counter
		cpx	#$20		; have we done 32 bytes?
		bne	Sled1		; not yet, get next character	
		lda	#$0D		; load CR code
		sta	Buffer+1	; ensure rest of text is ignored
		rts			; done
Sled_eol	lda	FONT_TABLE	; LED space chr font address pins
		sta	LbuffA, x	; save to disaply buffer
		lda	FONT_TABLE+1	; LED space chr font data pins
		sta	LbuffD, x	; save to display buffer
		inx			; inc display buffer
		cpx	#$20		; fill rest of buffer
		bne	Sled_eol	;	
		jmp	Monitor		; done, bypass Monitor command processing
					; and just start with a new line

;----------------------------------------------------------------------------
; call this to complete 1 digit scan.
; called from NMI, driven by 555 timer repeatedly to refresh entire display
;
;** NMI vector points to this routine
;
LED_scan	dec	Lscanc
		bne	Led_sfin3
		pha			; save a
		phx			; save x
		phy			; save y	
		lda	Lscan
		sta	Lscanc
		ldx	Ldig		; digit counter
		lda	LED_COL, x	; get Column mask
		sta	LSptr+1		; set A8-A10
		txa			; digit counter
		clc			; clear carry for add
		adc	Lptr		; add character pointer
		and	#$1F		; wrap if larger than 32 characters
		tax			; set up pointer to data
		lda	LbuffA, x	; get A0-A7 data pointer
		tay
		clv			; clear overflow flag (place before LED write)
		lda	LbuffD, x	; get D0-D7 data pointer
		sta	(LSptr),y	; write LED and scan button
		ldx	Ldig		; get button ptr
		bvs	Led_s1		; button pressed
                lda	Btn_bit0, x	; get bit
		and	btn_act		; mask bit
		sta	btn_act		; save
		bra	Led_sfin
Led_s1		lda	Btn_bit1, x	; get bit
		ora	btn_act		; set bit if button pressed
		sta	btn_act		; save it
		lda	Btn_bit1, x	; get bit
		ora	btn_ack		; set bit if button pressed
		sta	btn_ack		; save it
Led_sfin	lda	Ldig		; get LED pointer
		ina			; set to next LED
		and	#$07		; wrap if beyond 7
		sta	Ldig		; save it
		bne	Led_sfin2	; all done
		dec	Lsclc		; dec scroll counter
		bne	Led_sfin2	; not ready yet
		lda	Lscl		; done with scroll delay
		sta	Lsclc		; restore scroll counter	
		beq	Led_sfin2	; if scroll delay =0, don't scroll
		lda	Lptr		; get character pointer
		ina			; scroll one to the left
		and	#$1F		; mask to 32 characters
		sta	Lptr		; save it
Led_sfin2	ply			; restore y
		plx			; restore x
		pla			; restore a
;Led_sfin3	rti			; done, return from interrupt
Led_sfin3	rts			; done, return
;----------------------------------------------------------------------------
