; 6502 Badge Software
; Hardware design by Lee Hart
; Software design by Daryl Rictor
;
; This software is free to use and modify in any non-commercial application.
; Comericial use is prohibited without expressed, written permission from the author.
;

; defined and used in LEDdrive.asm
;btn_act		=	$58			; button's down if bit=1
;btn_ack		=	$59			; button ackowleged if bit=0

Btn_bit1	.byte	$01, $02, $04, $08, $10, $20, $40, $80
Btn_bit0	.byte	$FE, $FD, $FB, $F7, $EF, $DF, $BF, $7F

;----------------------------------------------------------------------------
; Call to see if a button is currently pressed (since last scan cycle)
; Send Button # in A, 0-7 range
; uses X register
; return C clear if not down, C Set if down

Check_Down	tax
		clc
		lda	Btn_bit1, x
		and	btn_act
		beq	chkdn1
		sec
chkdn1		rts

;----------------------------------------------------------------------------
; Call to see if a button was pressed but not acknowleged 
; Send Button # in A, 0-7 range
; uses X register
; return C clear if acknowleged, C Set if waiting action

Check_wait	tax
		clc
		lda	Btn_bit1, x
		and	btn_ack
		beq	chkwt1
		sec
chkwt1		rts

;----------------------------------------------------------------------------
; Acknowlege a button
; Does not validate if it was pressed or is being pressed
; Uses X register

Ackclr		tax
Ackclr1		lda	Btn_bit0, x
		and	btn_ack
		sta	btn_ack
		rts
;----------------------------------------------------------------------------
; Check if any button unacknowleged
; return C clear if no buttons waiting, C set if a button is waiting.
; returns button bit map in A

AnyBtn		lda	btn_ack
		clc
		beq	anybtn1
		sec
anybtn1		rts

;----------------------------------------------------------------------------
; Routine to update display
;

BTN_STRT	=	$05
BTN_SEL		=	$07
BTN_NXT		=	$02
BTN_PRV		=	$01
BTN_UP		=	$00
BTN_DN		=	$03


UpdateDisp	jsr	Ackclr1		; ack STRT button 
		lda	Lscl		; scroll speed
		pha			; save it
		lda	Lptr		; scroll pos		
		pha			; save it
		stz	Lscl		; stop scroll
		stz	Lptr		; reset scroll to 1st char
UpdDisp01	ldx	Lptr		; keep aligned with scroll
		lda	LbuffD,x	; get font D
		pha			; save	
		LDa	LbuffA,x	; get font A 
		pha			; save
		ldy	#$42		; "A" char
UpdDisp02	lda	FONT_TABLE,y
		sta	LbuffA,x
		lda	FONT_TABLE+1,y
		STA	LbuffD,x	; update display
		jsr	LED_scan	; keep display going
UpdDisp03	jsr	AnyBtn
		bcc	UpdDisp02	; wait for button
		bit	Btn_bit1+BTN_NXT	; next button
		beq	UpD1
		ldx	#BTN_NXT
		jsr	Ackclr1
		lda	Lptr
		ina	
UpD0		and	#$1F
		sta	Lptr
		pla
		sta	LbuffA,x
		pla
		sta	LbuffD,x	; restore char
		bra	UpdDisp01	; 
UpD1		bit	Btn_bit1+BTN_PRV
		beq	UpD2
		ldx	#BTN_PRV
		jsr	Ackclr1
		lda	Lptr
		dea	
		bra	UpD0
UpD2		bit	Btn_bit1+BTN_UP
		beq	Upd3
		ldx	#BTN_UP
		jsr	Ackclr1
		iny
		iny
		cpy	#$C0
		bne	UpD21
		ldy	#$00
UpD21		bra	UpdDisp02
Upd3		bit	Btn_bit1+BTN_DN
		beq	Upd4
		ldx	#BTN_DN
		jsr	Ackclr1
		dey
		dey
		cpy	#$FE
		bne	UpdDisp02
		ldy	#$BE
		bra	UpdDisp02
Upd4		bit	Btn_bit1+BTN_SEL
		beq	Upd5
		ldx	#BTN_SEL
		jsr	Ackclr1
		lda	Lptr
		ina	
		and	#$1F
		sta	Lptr
		pla			; discard old data
		pla
		jmp	UpdDisp01	;
Upd5		bit	Btn_bit1+BTN_STRT	;
		beq	UpdDisp03	; wait for next key
		ldx	#BTN_STRT
		jsr	Ackclr1
		pla
		sta	LbuffA,x
		PLA
		sta	LbuffD,x	; restore char
		pla			; 
		sta	Lptr
		pla	
		sta	Lscl
		rts
